"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = __importStar(require("react"));
var themed_settings_1 = require("@uifabricshared/themed-settings");
var theming_ramp_1 = require("@uifabricshared/theming-ramp");
var foundation_tokens_1 = require("@uifabricshared/foundation-tokens");
var theming_react_native_1 = require("@uifabricshared/theming-react-native");
/* tslint:disable-next-line no-any */
function getOptionsFromObj(obj) {
    var objType = obj && typeof obj;
    return ((objType === 'object' || objType === 'function') && obj.__composable) || undefined;
}
exports.getOptionsFromObj = getOptionsFromObj;
/**
 * Get the cache for the given component from the theme, creating it if necessary
 *
 * @param component - component to get the cache for, the component object itself will store the unique symbol for its lookups
 * @param theme - theme where the cache will be stored
 */
function _getComponentCache(cacheKey, theme) {
    theme[cacheKey] = theme[cacheKey] || {};
    return theme[cacheKey];
}
function _getSettingsFromTheme(theme, name) {
    return theming_ramp_1.getSettings(theme, name).settings;
}
function _getHasToken(slots) {
    var slotTokens = {};
    Object.keys(slots).forEach(function (slot) {
        var options = getOptionsFromObj(slots[slot].slotType);
        slotTokens[slot] = (options && options.resolvedTokens && options.resolvedTokens.tokenKeys) || undefined;
    });
    return function (target, key) {
        return slotTokens[target] && slotTokens[target].hasOwnProperty(key);
    };
}
function _nameFromSettings(styleSettings) {
    var settings = styleSettings.settings;
    var names = settings.filter(function (v) { return typeof v === 'string'; }).map(function (v) { return v; });
    return names && names.length > 0 ? names.join('-') : undefined;
}
function useStylingCore(props, options, baseKey, tokenCacheKey, lookupOverride) {
    // get the theme value from the context (or the default theme if it is not set)
    var theme = React.useContext(theming_react_native_1.ThemeContext) || theming_react_native_1.getTheme();
    // get the cache for this component from the theme
    var cache = _getComponentCache(tokenCacheKey, theme);
    // resolve the array of settings for these options
    lookupOverride = lookupOverride || props;
    var _a = themed_settings_1.getThemedSettings(options.settings, theme, cache, baseKey, lookupOverride, _getSettingsFromTheme), settings = _a.settings, key = _a.key;
    // finish by processing the tokens and turning IComponentSettings into ISlotProps (this removes things like _overrides)
    return theming_ramp_1.returnAsSlotProps(foundation_tokens_1.processTokens(props, theme, settings, options.resolvedTokens, key, cache));
}
/**
 * return a useStyling implementation, in the form of IUseComposeStyling, based on the passed in styleSettings.  The
 * styleSettings will be captured in the created closure and will be set up to enable the appropriate levels of caching.
 *
 * @param styleSettings - style settings to configure this function.  Note that this should be scoped to a single component.
 * @param name - optional base name to use as a cache key
 */
function initializeStyling(styleSettings, name) {
    // process the tokens and get them ready to render
    var slots = styleSettings.slots;
    styleSettings.resolvedTokens = foundation_tokens_1.buildComponentTokens(slots, _getHasToken(slots));
    // ensure we have a name to use for caching.  Try to pull something identifiable to help with debugging
    name = name || _nameFromSettings(styleSettings) || 'anonymous';
    var tokenCacheKey = Symbol(name);
    // create a useStyling implementation for this component type (per type, not per instance)
    return function (props, lookupOverride) {
        return useStylingCore(props, styleSettings, name, tokenCacheKey, lookupOverride);
    };
}
exports.initializeStyling = initializeStyling;
//# sourceMappingURL=useStyling.js.map